using System;
using System.Reflection;
using System.Data;
using System.Data.OleDb;
using System.Web;
using System.Web.UI;
using System.IO;
using System.Windows.Forms;

[assembly: AssemblyTitle("")]
[assembly: AssemblyDescription("")]
[assembly: AssemblyConfiguration("")]
[assembly: AssemblyCompany("")]
[assembly: AssemblyProduct("")]
[assembly: AssemblyCopyright("")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]
[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyKeyFile("")]

namespace wwwSharp.ClrHost.Examples
{
	public class DataAccess: MarshalByRefObject
	{
		const int maxStringLength = 255;
	
		private string m_ConnectionString = "";
		private string m_SqlQuery = "";
		
		public string ConnectionString
		{
			get {return m_ConnectionString;}
			set {m_ConnectionString = value;}
		}
		
		public string SqlQuery
		{
			get {return m_SqlQuery;}
			set {m_SqlQuery = value;}
		}
		
		public string ConvertQueryToHtml()
		{
			string result = "";
			OleDbConnection conn = new OleDbConnection(m_ConnectionString);
			try
			{
			    Cursor.Current = Cursors.WaitCursor;
			
				conn.Open();
				OleDbCommand cmd = new OleDbCommand(m_SqlQuery, conn);
				OleDbDataReader reader = cmd.ExecuteReader();
				try
				{
					result = WriteHtml(reader);
				}
				finally
				{				
					reader.Close();
				}
			}
			finally
			{
				conn.Close();
				Cursor.Current = Cursors.Default;
			}
			
			return result;
		}
		
		public string GetTableNamesForSelect()
		{
			string result = "";
			OleDbConnection conn = new OleDbConnection(m_ConnectionString);
			try
			{
			    Cursor.Current = Cursors.WaitCursor;
			
				conn.Open();
				DataTable tables = conn.GetOleDbSchemaTable(OleDbSchemaGuid.Tables,
					new object[] {null, null, null, "TABLE"});
				result = WriteTables(tables);
			}
			finally
			{
				conn.Close();
				Cursor.Current = Cursors.Default;
			}
			
			return result;
		}
		
		private string WriteHtml(IDataReader reader)
		{
			StringWriter strWriter = new StringWriter();
			HtmlTextWriter htmlWriter = new HtmlTextWriter(strWriter);
			Render(htmlWriter, reader);
			return strWriter.ToString();
		}
		
		protected static void RenderTHead(HtmlTextWriter writer, IDataReader reader)
		{
			DataTable metadata = reader.GetSchemaTable();
			
			writer.WriteFullBeginTag("thead");
			writer.WriteLine();
			writer.Indent++;
			writer.WriteFullBeginTag("tr");
			writer.WriteLine();
			writer.Indent++;
			foreach (DataRow row in metadata.Rows)
			{
				int colWidth = 10;
				Type colType = (Type)row["DataType"];
				if (colType == typeof(string))
				{
					colWidth = (int)row["ColumnSize"];
					if (colWidth < 6) colWidth = 10;
					if (colWidth > 50) colWidth = 50;
				}
				else if (colType == typeof(DateTime))
				{
					colWidth = 20;
				}
			
				writer.WriteBeginTag("th");
				writer.WriteAttribute("style", "width: " + (colWidth + 5).ToString() + "ex");
				writer.Write(HtmlTextWriter.TagRightChar);
				
				writer.Write(row["ColumnName"]);
				
				writer.WriteEndTag("th");
			}
			writer.Indent--;
			writer.WriteEndTag("tr");
			writer.Indent--;
			writer.WriteEndTag("thead");
		}
		
		protected static void Render(HtmlTextWriter writer, IDataReader reader) 
		{
			//Write table
			writer.WriteBeginTag("table");
			writer.WriteAttribute("id", "tblQuery");
			writer.WriteAttribute("border", "0");
			writer.Write(HtmlTextWriter.TagRightChar);
			writer.WriteLine();
			
			//Write tHead
			writer.Indent++;
			RenderTHead(writer, reader);
			writer.Indent--;
			writer.WriteLine();
	
			//Write tBody
			writer.Indent++;
			writer.WriteFullBeginTag("tbody");
			//Write rows
			writer.Indent++;
			while (reader.Read())			
			{
				writer.WriteFullBeginTag("tr");
				writer.WriteLine();
	
				writer.Indent++;
				for (int fld = 0; fld < reader.FieldCount; fld++) 
				{
					writer.WriteBeginTag("td");
					writer.Write(HtmlTextWriter.TagRightChar);
	
					if (reader.IsDBNull(fld))
					{
						writer.WriteBeginTag("span");
						writer.WriteAttribute("class", "nullValue");
						writer.Write(HtmlTextWriter.TagRightChar);
						
						writer.Write("&lt;NULL&gt;");
						
						writer.WriteEndTag("span");
					}
					else
					{
						TypeCode code = Type.GetTypeCode(reader.GetFieldType(fld));
						switch (code)
						{
							case TypeCode.Boolean:
							case TypeCode.Byte:
							case TypeCode.Char:
							case TypeCode.DateTime:
							case TypeCode.Decimal:
							case TypeCode.Double:
							case TypeCode.Int16:
							case TypeCode.Int32:
							case TypeCode.Int64:
							case TypeCode.SByte:
							case TypeCode.Single:
							case TypeCode.UInt16:
							case TypeCode.UInt32:
							case TypeCode.UInt64:
								writer.Write(reader.GetValue(fld).ToString());
								break;
							case TypeCode.String:
								string strValue = reader.GetString(fld);
								//Truncate long strings in this example.
								if (strValue.Length > maxStringLength)
									strValue = strValue.Substring(0, maxStringLength) + " ...";
								writer.Write(strValue);
								break;
							default:
								writer.WriteBeginTag("span");
								writer.WriteAttribute("class", "blobValue");
								writer.Write(HtmlTextWriter.TagRightChar);							
								
								writer.Write("&lt;BLOB&gt;");
								
								writer.WriteEndTag("span");
								
								break;
						}
					}
					writer.WriteEndTag("td");
					writer.WriteLine();
				}
				writer.Indent--;
				writer.WriteEndTag("tr");
				writer.WriteLine();
			}
			writer.Indent--;
			writer.WriteEndTag("tbody");
			writer.Indent--;
	
			writer.WriteEndTag("table");
			writer.WriteLine();
		}
		
		protected static string WriteTables(DataTable schema)
		{
			StringWriter strWriter = new StringWriter();
			HtmlTextWriter htmlWriter = new HtmlTextWriter(strWriter);
			
			DataView sortedSchema = new DataView(schema, 
				"TABLE_TYPE = 'TABLE' OR TABLE_TYPE = 'VIEW'",
				"TABLE_NAME", DataViewRowState.CurrentRows);
			
			RenderTables(htmlWriter, sortedSchema);
			return strWriter.ToString();
		}
		
		protected static void RenderTables(HtmlTextWriter writer, DataView schema)
		{
			for (int i = 0; i < schema.Count; i++)
			{
				string tblName = (string)schema[i]["TABLE_NAME"];
				
				writer.WriteBeginTag("option");
				writer.WriteAttribute("value", tblName);
				writer.Write(HtmlTextWriter.TagRightChar);
				
				writer.Write(tblName);
				
				writer.WriteEndTag("option");
				writer.WriteLine();
			}
		}
	}
}
